##!c:\perl\bin\perl -w
#
# Traitement des fichiers SeaBird CTD acquis avec Seasave et traits avec Seasoft
# SBEProcessing. 
# Utilise les fichiers d'extensions .hdr et .asc 
# Les fichiers de sortie prennent le nom du cycle_mesure (entete) + extensions 
# et _ctd + extension eventuelle .xml
#
# J Grelet IRD fevrier 2004/ decembre 2005
# 
# $Id: ctd-egee2.pl,v 1.4 2005/12/19 10:36:01 jgrelet Exp $ 

use strict; # necessite de declarer toutes les variables globales
#use diagnostics;

use Time::Local;
use Date::Manip;
use File::Basename;
use Data::Dumper;
use Getopt::Long;
use Switch;
use seawater;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION = '0.22c';
our $AUTHOR  = 'jgrelet IRD fevrier 2004 - avril 2005';
my  $DEBUG         = 0;
my  $ECHO;

# a mettre sous forme d'argument ligne de commande
my  $CONTEXTE = "AMMA";
my  $TIMEZONE = "GMT";
my  $FORMAT_DATE = "DMY";
my  $CODE = "2C";
my  $date_debut = "03/09/2005";
my  $date_fin   = "30/09/2005";

my  $version;
my  $help;
my  $xml     = undef;

my $code     = -1;  # code pour l'entete

my $hdr_file;
my $ctd_file;
my $institute;
my $ctdType;
my $inst_sn;
my $pi;

my($cruise, $ship) = undef;
my($seasave_version) = undef;
my($station, $depth,$T0,$T1,$chla,$ox1,$ox2,$nbin,$S0,$S1,$sigmateta,$sndvel );
my($lat, $lat_pos, $lat_deg, $lat_min, $lat_hemi);
my($long, $long_pos, $long_deg, $long_min, $long_hemi);
my($julien, $h_date, $date, $mois, $jour, $annee, $heure, $min, $time);

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: ctd-egee2.pl [options] <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug=[1-3]          debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "    --cruise=<name>        give cruise name\n";
  print STDERR   "    --ship=<ship_name>     ship name\n";
  print STDERR   "    --institute=<name>     institute name\n";
  print STDERR   "    --pi=<pi_name> \n";
  print STDERR   "    --xml                  Force output to XML\n";
  print STDERR   "    --instrument=<serial_number>\n";
  print STDERR   "    --type=<instrument_type> \n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "examples:\n\$ perl ctd-egee2.pl --cruise=EGEE2 --institute=IRD --ship='LE SUROIT' --instrument=09P10828 --type=SBE911+ --pi=Grelet data/ege2*.HDR --xml --echo\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("cruise=s"        => \$cruise,    
               "type=s"          => \$ctdType,  
               "instrument=s"    => \$inst_sn,  
               "ship=s"          => \$ship,  
               "pi=s"            => \$pi,  
               "xml"             => \$xml,  
               "debug=i"         => \$DEBUG,  
               "echo"            => \$ECHO,  	       
               "institute=s"     => \$institute,  
               "version"         => \$version,  
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;	
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi) = @_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
sub julian {
  my($jj,$h,$m) = @_;
  my $tmp = (($h * 60) + $m ) / 1440;
  return( $jj + $tmp ); 
}

#------------------------------------------------------------------------------
# voir documentation 
# http://www.seabird.com/pdf_documents/manuals/Seasave_5.37.pdf
#------------------------------------------------------------------------------
sub decode_con_file {
  my($fileName) = @_;
  my($p_cond_g,$p_cond_h,$p_cond_i,$p_cond_j,$p_cond_ctcor,$p_cond_cpcor);
  my($p_cond_sn,$p_cond_date,$p_cond_slope,$p_cond_offset);
  my($p_temp_f0,$p_temp_g,$p_temp_h,$p_temp_i,$p_temp_j);
  my($p_temp_sn,$p_temp_date,$p_temp_slope,$p_temp_offset);

  # attention, verifier si les fichiers sont en majuscules ou minuscules !!!!
  $fileName =~ s/\.HDR/\.CON/;
  open( DATA_CON, $fileName );
  while( <DATA_CON> ){           
    chomp;
    switch( $. ) {
       # conductivity	    
       case 1   { $p_cond_sn =  $_; }
       case 3   { (undef,undef,$p_cond_slope,$p_cond_offset) = split; }
       case 110 { ($p_cond_g,$p_cond_h,$p_cond_i,$p_cond_j,$p_cond_ctcor,$p_cond_cpcor) = split; }
       case 52  { $p_cond_date =  $_; }
       # temperature
       case 4   { $p_temp_sn =  $_; }
       case 5   { (undef,undef,undef,undef,undef,$p_temp_slope,$p_temp_offset) = split; }
       case 111 { ($p_temp_f0,$p_temp_g,$p_temp_h,$p_temp_i,$p_temp_j) = split; }
       case 53  { $p_temp_date =  $_; }
    }  
  }
  close DATA_CON;

  #my $xml = qq|      
  #        <ETALONNAGE DATE=\"$p_cond_date\" CAPTEUR=\"COND\" NUMERO_SERIE=\"$p_cond_sn\" NUMERO=\"PRIMARY\">  
  #        <PARAMETRE VALEUR=\"G\">$p_cond_g</PARAMETRE>
  #|;
  #print CTD_FILE $xml;
  print CTD_FILE "    <INSTRUMENT TYPE=\"SBE04\" NUMERO_SERIE=\"$p_cond_sn\" CODE_ROSCOP=\"PSAL\" PERE=\"$inst_sn\" NUMERO=\"PRIMARY\">\n"; 
  print CTD_FILE "      <PARAMETRE CLE=\"DATE\" TYPE=\"D\">$p_cond_date</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"H\" TYPE=\"N\">$p_cond_h</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"I\" TYPE=\"N\">$p_cond_i</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"J\" TYPE=\"N\">$p_cond_j</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"CTCOR\" TYPE=\"N\">$p_cond_ctcor</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"CPCOR\" TYPE=\"N\">$p_cond_cpcor</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"SLOPE\" TYPE=\"N\">$p_cond_slope</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"OFFSET\" TYPE=\"N\">$p_cond_offset</PARAMETRE>\n";
  print CTD_FILE "    </INSTRUMENT>\n";  
  print CTD_FILE "    <INSTRUMENT TYPE=\"SBE03\" NUMERO_SERIE=\"$p_temp_sn\" CODE_ROSCOP=\"TEMP\" PERE=\"$inst_sn\" NUMERO=\"PRIMARY\">\n"; 
  print CTD_FILE "      <PARAMETRE CLE=\"DATE\" TYPE=\"D\">$p_temp_date</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"G\" TYPE=\"N\">$p_temp_g</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"H\" TYPE=\"N\">$p_temp_h</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"I\" TYPE=\"N\">$p_temp_i</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"J\" TYPE=\"N\">$p_temp_j</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"F0\" TYPE=\"N\">$p_temp_f0</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"SLOPE\" TYPE=\"N\">$p_temp_slope</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"OFFSET\" TYPE=\"N\">$p_temp_offset</PARAMETRE>\n";
  print CTD_FILE "    </INSTRUMENT>\n";  
}

#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&Date_Init( "TZ=UTC" );
&usage if( $#ARGV == -1);
&get_options;

my $today = &UnixDate( &ParseDate("today"), "%d/%m/%Y");

if( $DEBUG ) {
  printf "%s\n%s\n%s\n%s\n%s\n%s\n%s\n",
          $hdr_file,$ctd_file,$institute,$ship,$ctdType,$inst_sn,$pi;
  for( my $i = 0; $i <= $#ARGV; $i++ ){
    print $ARGV[$i] . "\n";
  }
  print "\nExit ctd.pl ... debug mode\n";
  exit;
}

# lecture du premier fichier d'entete pour extraction des parametres
# generaux si ces derniers n'ont pas ete definit sur la ligne de commande.
open( DATA_FILE, $ARGV[0] );
while( <DATA_FILE> ){           # header contient l'entete
  if( /Ship\s*:\s*(.*)/) {
    if ( not defined $ship ) {
      ($ship) = $1;
       chop $ship;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ ) {
    ($cruise) = $1 if ( not defined $cruise );
  }
  if( /Sea-Bird\s+(\w+\s+\d+)/ ) {
    ($ctdType) = $1  if ( not defined $ctdType );
  }
  if( /Software Version Seasave Win32 V\s+(\d+\.\d+)/ ) {
    ($seasave_version) = $1  if ( not defined $seasave_version );
  }
}  

# ouverture des fichiers de sortie, on met en minuscule, lower case
my $fileName =  $ARGV[0];
my ($name,$dir) =  fileparse $fileName;
$ctd_file = lc $cruise.'_ctd';
$ctd_file .= '.xml' if defined $xml;
$hdr_file = lc  $cruise.'.ctd';
open( CTD_FILE, "+> $ctd_file" ) or die "Can't open file : $ctd_file\n";
open( HDR_FILE, "+> $hdr_file" ) or die "Can't open file : $hdr_file\n";

# ecriture des entetes
if ( defined $xml ) {
  print  CTD_FILE '<?xml version="1.0" encoding="ISO-8859-1"?>' . "\n"; 
  print  CTD_FILE '<!--' . "\n";
  print  CTD_FILE '<!DOCTYPE OCEANO SYSTEM "oceano.dtd">' . "\n";
  print  CTD_FILE '-->' . "\n";
  print  CTD_FILE '<!DOCTYPE OCEANO PUBLIC "-//US025//DTD OCEANO//FR" "http://www.brest.ird.fr/us025/database/oceano.dtd">' . "\n";
  
  print  CTD_FILE '<OCEANO TYPE="PROFIL">' . "\n";
  print  CTD_FILE "  <ENTETE>\n";
  print  CTD_FILE "    <PLATEFORME>\n";
  print  CTD_FILE "      <LIBELLE>$ship</LIBELLE>\n";  
  print  CTD_FILE "    </PLATEFORME>\n";
  print  CTD_FILE "    <CYCLE_MESURE CONTEXTE=\"$CONTEXTE\" TIMEZONE=\"$TIMEZONE\" FORMAT=\"$FORMAT_DATE\">\n";  
  print  CTD_FILE "      <LIBELLE>$cruise</LIBELLE>\n";  
  print  CTD_FILE "      <DATE_DEBUT>$date_debut</DATE_DEBUT>\n";  
  print  CTD_FILE "      <DATE_FIN>$date_fin</DATE_FIN>\n";  
  print  CTD_FILE "      <INSTITUT>$institute</INSTITUT>\n";  
  print  CTD_FILE "      <RESPONSABLE>$pi</RESPONSABLE>\n"; 
  print  CTD_FILE "      <ACQUISITION LOGICIEL=\"SEASAVE\" VERSION=\"$seasave_version\"></ACQUISITION>\n"; 
  print  CTD_FILE "      <TRAITEMENT LOGICIEL=\"SBEDATAPROCESSING\" VERSION=\"$seasave_version\"></TRAITEMENT>\n"; 
  print  CTD_FILE "      <VALIDATION LOGICIEL=\"$0\" VERSION=\"$VERSION\" DATE=\"$today\" OPERATEUR=\"$pi\" CODIFICATION=\"OOPC\">\n";
  print  CTD_FILE "        <CODE>$CODE</CODE>\n";	    
  print  CTD_FILE "        <COMMENTAIRE>Extraction ralise avant la post-calibration</COMMENTAIRE>\n";
  print  CTD_FILE "      </VALIDATION>\n";  
  print  CTD_FILE "    </CYCLE_MESURE>\n";  
  print  CTD_FILE "    <INSTRUMENT TYPE=\"$ctdType\" NUMERO_SERIE=\"$inst_sn\">\n"; 
  print  CTD_FILE "    </INSTRUMENT>\n";  
  &decode_con_file($fileName);
  print  CTD_FILE "  </ENTETE>\n";  
  print  CTD_FILE "  <DATA>\n";  
} else {	
  print CTD_FILE "$cruise  $ship  $institute  $ctdType  $inst_sn  $pi\n";
}
print CTD_FILE "ST    PRES  TEMP    PSAL    DENS     SVEL     DOX2    FLU2   N/A\n";
print HDR_FILE "$cruise  $ship  $institute  $ctdType  $inst_sn  $pi\n";
print HDR_FILE "St  Heure   Date    Latitude   Longitude  Profondeur\n\n";
close DATA_FILE;

# parcourt des fichiers .HDR et .asc
for( my $i = 0; $i <= $#ARGV; $i++ ){
  my $fileName = $ARGV[$i];
  open( DATA_FILE, $fileName );
  print STDERR  "Lit: $fileName" if defined $ECHO;
  # on lit les fichiers d'extention .hdr contenant les entetes
  while( <DATA_FILE> ){           
    if( /Station\s*:\s*(\d+)/ )	{
      ($station)  = $1; 
    }
    if( /System UpLoad Time =\s+(\.*)/)	{ # a modifier suivant le contexte
    ($time)  = /System UpLoad Time =\s+(\w+\s+\d+\s+\d+\s+\d+:\d+:\d+)/;	
    $date=&ParseDate($time);
    # transforme le day_of_year en julian day
    $julien = &UnixDate($time,"%j") -1;
    $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M") );
    $h_date = &UnixDate($date,"%H:%M %d/%m/%Y");
    }
    #if( m[Date\s*:\s*(\d+)/(\d+)/(\d+)]) {
    #  ($jour,$mois,$annee) = ($1, $2, $3);	
    #};	
    #if( /Heure\s*:\s*(\d+)[:hH](\d+)/ ) {  # a modifier suivant le contexte
    #  ($heure,$min)  = ($1, $2);	
    #}
    #if( /Latitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ )	{
    #  ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
    #  $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    #}
    #if( /Longitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ ) {
    #  ($long_deg, $long_min, $long_hemi) = split " ", $1;
    #  $long_pos = &position($long_deg, $long_min, $long_hemi);
    #}
    if( /NMEA Latitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /NMEA Longitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
  }
  # conversion en format US (MM/DD/YY) pour ParseDate
  #$date = $mois . "/" . $jour . "/" . $annee . " " . $heure . ":" . $min . ":00";
  #print STDERR  "Date: $date\n";
  #$date = &ParseDate( $date );
  ## transforme le day_of_year en julian day
  #$julien = &UnixDate($time,"%j") -1;
  #$julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M") );
  #$h_date=&UnixDate($date,"%H:%M %d/%m/%Y");
  printf CTD_FILE "%3d  %4d %7.3f %7.4f %8.4f %s 1e36 1e36 1e36\n",$station,
         $code,$julien,$lat_pos, $long_pos,&UnixDate($date,"%q");
  close DATA_FILE;
  # on lit les fichiers d'extension .asc contenant les donnees
  # attention aux majuscules/minuscules !!!!!
  $fileName =~ s/\.HDR/\.asc/;
  open( DATA_FILE, $fileName );
  while( <DATA_FILE> ){           
    # on expurge toutes les lignes des entetes ainsi que les lignes vides
    # n'est pas necessaire avec asciiout
    if( not ( /^[*#]/ || /^\s*$/) ) {
      ($depth,$T0,undef,$chla,$ox1,undef,$nbin,$S0,undef,$sigmateta,$sndvel) = split;
      #($depth,$T0,$T1,$chla,$ox1,$ox2,$nbin,$S0,$S1,$sigmateta,$sndvel) = split;
      next if( $depth eq 'PrDM' ); 
      #$T68 = $T0 * 1.00024; 
      #$sndvel=&sw_svel($S0,$T68,$depth);
      printf CTD_FILE "%3d  %4d  %6.3f  %6.3f  %6.3f  %7.2f  %7.6g  %+7.3g  %3d\n",
         $station,$depth,$T0,$S0,$sigmateta,$sndvel,$ox1,$chla,$nbin;
    }
  }
  printf HDR_FILE  "%03d %s %02d%05.2f %s %03d%05.2f %s %4.4g\n",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi, $depth;
  printf STDERR "   %03d %s %02d%05.2f %s %03d%05.2f %s %5.4g\n",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi, $depth if defined $ECHO;
  $depth = 1e36;	  
  $. = 0; # remet le compteur de ligne a zero
  close DATA_FILE;
}

# return result to matlab
print STDERR $ctd_file;

close HDR_FILE;
if ( defined $xml ) {
  print  CTD_FILE "  </DATA>\n";  
  print  CTD_FILE "</OCEANO>\n";  
}  
close CTD_FILE;

